@@ -30,6 +30,7 @@ class Commands(discord.Cog, name="Commands"):
         translations_cog=None,
         search_cog=None,
         transcribe_cog=None,
+        code_interpreter_cog=None,
     ):
         super().__init__()
         self.bot = bot
@@ -45,6 +46,7 @@ class Commands(discord.Cog, name="Commands"):
         self.translations_cog = translations_cog
         self.search_cog = search_cog
         self.transcribe_cog = transcribe_cog
+        self.code_interpreter_cog = code_interpreter_cog
 
     # Create slash command groups
     dalle = discord.SlashCommandGroup(
@@ -87,7 +89,15 @@ class Commands(discord.Cog, name="Commands"):
         name="internet",
         description="Transcription services using OpenAI Whisper2",
         guild_ids=ALLOWED_GUILDS,
-        checks=[Check.check_index_roles()],  # TODO new role checker for transcribe
+        checks=[Check.check_index_roles()],  # TODO new role checker for internet
+    )
+    code_interpreter = discord.SlashCommandGroup(
+        name="code",
+        description="Code interpreter functionalities",
+        guild_ids=ALLOWED_GUILDS,
+        checks=[
+            Check.check_index_roles()
+        ],  # TODO new role checker for code interpreter
     )
 
     #
@@ -724,39 +734,25 @@ class Commands(discord.Cog, name="Commands"):
 
     @add_to_group("index")
     @discord.slash_command(
-        name="talk",
+        name="chat",
         description="Select one of your saved indexes to talk to",
         guild_ids=ALLOWED_GUILDS,
     )
     @discord.guild_only()
-    @discord.option(
-        name="user_index",
-        description="Which user file to load the index from",
-        required=False,
-        autocomplete=File_autocompleter.get_user_indexes,
-    )
-    @discord.option(
-        name="search_index",
-        description="Which search index file to load the index from",
-        required=False,
-        autocomplete=File_autocompleter.get_user_search_indexes,
-    )
     @discord.option(
         name="model",
         description="The model to use for the conversation",
         required=False,
-        default="gpt-3.5-turbo",
-        autocomplete=Settings_autocompleter.get_index_and_search_models,
+        default="gpt-4-32k",
+        autocomplete=Settings_autocompleter.get_function_calling_models,
     )
     async def talk(
         self,
         ctx: discord.ApplicationContext,
-        user_index: str,
-        search_index: str,
         model: str,
     ):
         await ctx.defer()
-        await self.index_cog.index_chat_command(ctx, user_index, search_index, model)
+        await self.index_cog.index_chat_command(ctx, model)
 
     @add_to_group("index")
     @discord.slash_command(
@@ -914,7 +910,7 @@ class Commands(discord.Cog, name="Commands"):
         name="model",
         description="The model to use for the request (querying, not composition)",
         required=False,
-        default="gpt-3.5-turbo",
+        default="gpt-4-32k",
         autocomplete=Settings_autocompleter.get_index_and_search_models,
     )
     @discord.option(
@@ -951,13 +947,51 @@ class Commands(discord.Cog, name="Commands"):
 
     @add_to_group("dalle")
     @discord.slash_command(
-        name="draw",
-        description="Draw an image from a prompt",
+        name="draw_old",
+        description="Draw an image from a prompt using the old DALLE-2 Model",
         guild_ids=ALLOWED_GUILDS,
     )
     @discord.option(name="prompt", description="The prompt to draw from", required=True)
-    async def draw(self, ctx: discord.ApplicationContext, prompt: str):
-        await self.image_draw_cog.draw_command(ctx, prompt)
+    async def draw_old(self, ctx: discord.ApplicationContext, prompt: str):
+        await self.image_draw_cog.draw_old_command(ctx, prompt)
+
+    @add_to_group("dalle")
+    @discord.slash_command(
+        name="draw",
+        description="Draw an image from a prompt using the new DALLE-3 Model. Does not support Variations.",
+        guild_ids=ALLOWED_GUILDS,
+    )
+    @discord.option(name="prompt", description="The prompt to draw from", required=True)
+    @discord.option(
+        name="quality",
+        description="Image quality",
+        required=False,
+        default="hd",
+        autocomplete=Settings_autocompleter.get_dalle3_image_qualities,
+    )
+    @discord.option(
+        name="image_size",
+        description="How big you want the generated image to be",
+        required=False,
+        default="1024x1024",
+        autocomplete=Settings_autocompleter.get_dalle3_image_sizes,
+    )
+    @discord.option(
+        name="style",
+        description="The style of the generated images, choose between realism/vivid",
+        required=False,
+        default="natural",
+        autocomplete=Settings_autocompleter.get_dalle3_image_styles,
+    )
+    async def draw(
+        self,
+        ctx: discord.ApplicationContext,
+        prompt: str,
+        quality: str,
+        image_size: str,
+        style: str,
+    ):
+        await self.image_draw_cog.draw_command(ctx, prompt, quality, image_size, style)
 
     @add_to_group("dalle")
     @discord.slash_command(
@@ -1021,6 +1055,36 @@ class Commands(discord.Cog, name="Commands"):
     async def draw_action(self, ctx, message: discord.Message):
         await self.image_draw_cog.draw_action(ctx, message)
 
+    """
+    Code interpreter commands and actions
+    """
+
+    @add_to_group("code")
+    @discord.slash_command(
+        name="chat",
+        description="Chat with code-interpreting GPT!",
+        guild_ids=ALLOWED_GUILDS,
+        checks=[Check.check_search_roles()],
+    )
+    @discord.option(
+        name="model",
+        description="The model to use for the request (querying, not composition)",
+        required=False,
+        default="gpt-4-32k",
+        autocomplete=Settings_autocompleter.get_function_calling_models,
+    )
+    async def chat_code(
+        self,
+        ctx: discord.ApplicationContext,
+        model: str,
+    ):
+        if not self.code_interpreter_cog:
+            await ctx.respond(
+                "Code interpretation is disabled on this server.", ephemeral=True
+            )
+            return
+        await self.code_interpreter_cog.code_interpreter_chat_command(ctx, model=model)
+
     """
     Translation commands and actions
     """
@@ -1133,8 +1197,8 @@ class Commands(discord.Cog, name="Commands"):
         name="model",
         description="The model to use for the request (querying, not composition)",
         required=False,
-        default="gpt-3.5-turbo",
-        autocomplete=Settings_autocompleter.get_index_and_search_models,
+        default="gpt-4-32k",
+        autocomplete=Settings_autocompleter.get_function_calling_models,
     )
     async def chat(
         self,
@@ -1189,7 +1253,7 @@ class Commands(discord.Cog, name="Commands"):
         name="model",
         description="The model to use for the request (querying, not composition)",
         required=False,
-        default="gpt-3.5-turbo",
+        default="gpt-4-32k",
         autocomplete=Settings_autocompleter.get_index_and_search_models,
     )
     @discord.option(
