@@ -13,38 +13,32 @@ import discord
 import openai
 from bs4 import BeautifulSoup
 from discord.ext import pages
-from langchain import (
+from langchain.utilities import (
     GoogleSearchAPIWrapper,
-    WolframAlphaAPIWrapper,
-    FAISS,
-    InMemoryDocstore,
-    LLMChain,
-    ConversationChain,
 )
+from langchain.utilities import WolframAlphaAPIWrapper
 from langchain.agents import (
     Tool,
     initialize_agent,
     AgentType,
-    ZeroShotAgent,
-    AgentExecutor,
 )
 from langchain.chat_models import ChatOpenAI
-from langchain.memory import ConversationBufferMemory, CombinedMemory
-from langchain.prompts import (
-    ChatPromptTemplate,
-    SystemMessagePromptTemplate,
-    MessagesPlaceholder,
-    HumanMessagePromptTemplate,
+from langchain.memory import (
+    ConversationSummaryBufferMemory,
 )
-from langchain.requests import TextRequestsWrapper, Requests
+from langchain.prompts import (
+    MessagesPlaceholder,
+)
+from langchain.requests import Requests
+from langchain.schema import SystemMessage
 from llama_index import (
     GPTVectorStoreIndex,
     Document,
     SimpleDirectoryReader,
     ServiceContext,
     OpenAIEmbedding,
-    ResponseSynthesizer,
 )
+from llama_index.response_synthesizers import get_response_synthesizer, ResponseMode
 from llama_index.retrievers import VectorIndexRetriever
 from llama_index.query_engine import RetrieverQueryEngine
 from llama_index.prompts.chat_prompts import CHAT_REFINE_PROMPT
@@ -186,6 +180,11 @@ class CustomTextRequestWrapper(BaseModel):
             original_query = "No Original Query Provided"
 
         """GET the URL and return the text."""
+        if not url.startswith("http"):
+            return (
+                "The website could not be crawled as an invalid URL was input. The input URL was "
+                + url
+            )
         text = self.requests.get(url, **kwargs).text
 
         # Load this text into BeautifulSoup, clean it up and only retain text content within <p> and <title> and <h1> type tags, get rid of all javascript and css too.
@@ -224,8 +223,8 @@ class CustomTextRequestWrapper(BaseModel):
                 retriever = VectorIndexRetriever(
                     index=index, similarity_top_k=4, service_context=service_context
                 )
-                response_synthesizer = ResponseSynthesizer.from_args(
-                    response_mode="compact",
+                response_synthesizer = get_response_synthesizer(
+                    response_mode=ResponseMode.COMPACT,
                     refine_template=CHAT_REFINE_PROMPT,
                     service_context=service_context,
                     use_async=True,
@@ -301,31 +300,6 @@ class SearchService(discord.Cog, name="SearchService"):
 
         return pages
 
-    async def paginate_chat_embed(self, response_text):
-        """Given a response text make embed pages and return a list of the pages."""
-
-        response_text = [
-            response_text[i : i + 3500] for i in range(0, len(response_text), 7000)
-        ]
-        pages = []
-        first = False
-        # Send each chunk as a message
-        for count, chunk in enumerate(response_text, start=1):
-            if not first:
-                page = discord.Embed(
-                    title=f"{count}",
-                    description=chunk,
-                )
-                first = True
-            else:
-                page = discord.Embed(
-                    title=f"{count}",
-                    description=chunk,
-                )
-            pages.append(page)
-
-        return pages
-
     @discord.Cog.listener()
     async def on_message(self, message):
         # Check if the message is from a bot.
@@ -336,6 +310,10 @@ class SearchService(discord.Cog, name="SearchService"):
         if not message.guild:
             return
 
+        # System message
+        if message.type != discord.MessageType.default:
+            return
+
         if message.content.strip().startswith("~"):
             return
 
@@ -412,13 +390,14 @@ class SearchService(discord.Cog, name="SearchService"):
                 return
 
             if len(response) > 2000:
-                embed_pages = await self.paginate_chat_embed(response)
-                paginator = pages.Paginator(
-                    pages=embed_pages,
-                    timeout=None,
-                    author_check=False,
-                )
-                await paginator.respond(message)
+                embed_pages = EmbedStatics.paginate_chat_embed(response)
+
+                for x, page in enumerate(embed_pages):
+                    if x == 0:
+                        previous_message = await message.reply(embed=page)
+                    else:
+                        previous_message = previous_message.reply(embed=page)
+
             else:
                 response = response.replace("\\n", "\n")
                 # Build a response embed
@@ -444,9 +423,9 @@ class SearchService(discord.Cog, name="SearchService"):
             description=f"The agent will visit and browse **{search_scope}** link(s) every time it needs to access the internet.\nCrawling is enabled, send the bot a link for it to access it!\nModel: {model}\n\nType `end` to stop the conversation",
             color=0xBA6093,
         )
-        message_embed.set_thumbnail(url="https://i.imgur.com/lt5AYJ9.png")
+        message_embed.set_thumbnail(url="https://i.imgur.com/sioynYZ.png")
         message_embed.set_footer(
-            text="Internet Chat", icon_url="https://i.imgur.com/lt5AYJ9.png"
+            text="Internet Chat", icon_url="https://i.imgur.com/sioynYZ.png"
         )
         message_thread = await ctx.send(embed=message_embed)
         thread = await message_thread.create_thread(
@@ -454,7 +433,6 @@ class SearchService(discord.Cog, name="SearchService"):
             auto_archive_duration=60,
         )
         await ctx.respond("Conversation started.")
-        print("The search scope is " + str(search_scope) + ".")
 
         # Make a new agent for this user to chat.
         search = GoogleSearchAPIWrapper(
@@ -475,7 +453,7 @@ class SearchService(discord.Cog, name="SearchService"):
             Tool(
                 name="Web-Crawling-Tool",
                 func=requests.get,
-                description=f"Useful for when the user provides you with a website link, use this tool to crawl the website and retrieve information from it. The input to this tool is a comma separated list of three values, the first value is the link to crawl for, and the second value is {model} and is the GPT model used, and the third value is the original question that the user asked. For example, an input could be 'https://google.com', gpt-3.5-turbo, 'What is this webpage?'. This tool should only be used if a direct link is provided and not in conjunction with other tools.",
+                description=f"Useful for when the user provides you with a website link, use this tool to crawl the website and retrieve information from it. The input to this tool is a comma separated list of three values, the first value is the link to crawl for, and the second value is {model} and is the GPT model used, and the third value is the original question that the user asked. For example, an input could be 'https://google.com', gpt-4-32k, 'What is this webpage?'. This tool should only be used if a direct link is provided and not in conjunction with other tools. The link should always start with http or https.",
             ),
         ]
 
@@ -494,21 +472,33 @@ class SearchService(discord.Cog, name="SearchService"):
             traceback.print_exc()
             print("Wolfram tool not added to internet-connected conversation agent.")
 
-        memory = ConversationBufferMemory(
-            memory_key="chat_history", return_messages=True
-        )
-
         llm = ChatOpenAI(model=model, temperature=0, openai_api_key=OPENAI_API_KEY)
 
+        max_token_limit = 29000 if "gpt-4" in model else 7500
+
+        memory = ConversationSummaryBufferMemory(
+            memory_key="memory",
+            return_messages=True,
+            llm=llm,
+            max_token_limit=100000 if "preview" in model else max_token_limit,
+        )
+
+        agent_kwargs = {
+            "extra_prompt_messages": [MessagesPlaceholder(variable_name="memory")],
+            "system_message": SystemMessage(
+                content="You are a superpowered version of GPT-4 that is able to access the internet. You can use google search to browse the web, you can crawl the web to see the content of specific websites, and in some cases you can also use Wolfram Alpha to perform mathematical operations. Use all of these tools to your advantage. You can use tools multiple times, for example if asked a complex question, search multiple times for different pieces of info until you achieve your goal."
+            ),
+        }
+
         agent_chain = initialize_agent(
-            tools,
-            llm,
-            agent=AgentType.CHAT_CONVERSATIONAL_REACT_DESCRIPTION,
+            tools=tools,
+            llm=llm,
+            agent=AgentType.OPENAI_FUNCTIONS,
             verbose=True,
+            agent_kwargs=agent_kwargs,
             memory=memory,
-            max_execution_time=120,
-            max_iterations=4,
-            early_stopping_method="generate",
+            handle_parsing_errors="Check your output and make sure it conforms!",
+            max_iterations=5,
         )
 
         self.chat_agents[thread.id] = agent_chain
@@ -745,4 +735,5 @@ class FollowupModal(discord.ui.Modal):
             from_followup=FollowupData(message_link, self.children[0].value),
             response_mode=self.search_cog.redo_users[self.ctx.user.id].response_mode,
             followup_user=interaction.user,
+            model="gpt-4-32k",
         )
