@@ -278,15 +278,21 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
 
         print("Syncing commands...")
 
-        await self.bot.sync_commands(
-            commands=None,
-            method="individual",
-            force=True,
-            guild_ids=ALLOWED_GUILDS,
-            register_guild_commands=True,
-            check_guilds=[],
-            delete_existing=True,
-        )
+        try:
+            await self.bot.sync_commands(
+                commands=None,
+                method="individual",
+                force=True,
+                guild_ids=ALLOWED_GUILDS,
+                register_guild_commands=True,
+                check_guilds=[],
+                delete_existing=True,
+            )
+        except:
+            traceback.print_exc()
+            print(
+                "There was a failure during command syncing. This might mean that the bot may not be in a guild that it expects to be in anymore, or it is in guilds without the guild ID being added to ALLOWED_GUILDS."
+            )
         print("Commands synced")
 
         # Start an inline async loop that runs every 10 seconds to save the conversation history to a pickle file
@@ -602,7 +608,7 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
     async def summarize_conversation(self, message, prompt):
         """Takes a conversation history filled prompt and summarizes it to then start a new history with it as the base"""
         response = await self.model.send_summary_request(prompt)
-        summarized_text = response["choices"][0]["text"]
+        summarized_text = response["choices"][0]["message"]["content"]
 
         new_conversation_history = []
         new_conversation_history.append(
@@ -660,6 +666,7 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
         # Moderations service is done here.
         if (
             hasattr(message, "guild")
+            and message.guild
             and message.guild.id in Moderation.moderation_queues
             and Moderation.moderation_queues[message.guild.id] is not None
         ):
@@ -691,8 +698,13 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
 
         # Process the message if the user is in a conversation
         if await TextService.process_conversation_message(
-            self, message, USER_INPUT_API_KEYS, USER_KEY_DB, file=file
+            self,
+            message,
+            USER_INPUT_API_KEYS,
+            USER_KEY_DB,
+            files=None if not file else message.attachments,
         ):
+            print("Processing a conversation message in server", message.guild.name)
             original_message[message.author.id] = message.id
 
         # If the user tagged the bot and the tag wasn't an @here or @everyone, retrieve the message
@@ -760,43 +772,55 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
         """Command handler. Generates a help message and sends it to the user"""
         await ctx.defer()
         embed = discord.Embed(
-            title="GPT3Bot Help", description="The current commands", color=0xC730C7
+            title="GPTDiscord Help", description="The current commands", color=0xC730C7
         )
         embed.add_field(
-            name="/search",
-            value="AI-Assisted google search!",
+            name="/code chat",
+            value="Chat with GPT hooked up to a full execution environment!",
+            inline=False,
+        )
+        embed.add_field(
+            name="/internet chat",
+            value="AI-Assisted chatting connected to google, wolfram, and a crawler!",
+            inline=False,
+        )
+        embed.add_field(
+            name="/internet search",
+            value="Do a one-off internet search assisted by GPT!",
+            inline=False,
+        )
+        embed.add_field(
+            name="/index chat",
+            value="Chat with your documents",
             inline=False,
         )
         embed.add_field(
             name="/index",
-            value="Indexing commands for document knowledge and querying",
+            value="More indexing/working with documents commands",
+            inline=False,
+        )
+        embed.add_field(
+            name="/gpt converse",
+            value="Have a multi-modal conversation with GPT",
             inline=False,
         )
         embed.add_field(
             name="/gpt ask",
-            value="Ask GPT something. Be clear, long, and concise in your prompt. Don't waste tokens.",
+            value="Ask GPT something. Be clear, long, and concise in your prompt.",
             inline=False,
         )
         embed.add_field(
-            name="/gpt edit",
-            value="Use GPT to edit a piece of text given an instruction",
+            name="/gpt",
+            value="More pure GPT commands",
             inline=False,
         )
         embed.add_field(
-            name="/gpt converse", value="Start a conversation with GPT", inline=False
-        )
-        embed.add_field(
-            name="/gpt end",
-            value="End a conversation with GPT. You can also type `end` in the conversation.",
-            inline=False,
-        )
-        embed.add_field(
-            name="/dalle draw <image prompt>",
+            name="/dalle draw",
             value="Use DALL-E2 to draw an image based on a text prompt",
             inline=False,
         )
         embed.add_field(
-            name="/dalle optimize <image prompt>",
+            name="/dalle optimize",
             value="Optimize an image prompt for use with DALL-E2, Midjourney, SD, etc.",
             inline=False,
         )
@@ -820,6 +844,11 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
             value="Translate from one language to another",
             inline=False,
         )
+        embed.add_field(
+            name="/transcribe",
+            value="Transcription services (very experimental)",
+            inline=False,
+        )
 
         embed.add_field(name="/help", value="See this help text", inline=False)
         await ctx.respond(embed=embed, ephemeral=False)
@@ -1150,14 +1179,14 @@ class GPT3ComCon(discord.Cog, name="GPT3ComCon"):
                     description=f"**Model**: {self.model.model if not model else model}",
                     color=0x808080,
                 )
-                message_embed.set_thumbnail(url="https://i.imgur.com/asA13vI.png")
+                message_embed.set_thumbnail(url="https://i.imgur.com/wpp4TIf.png")
                 footer_text = (
                     "Regular Chat"
                     if not image_understanding_model.get_is_usable()
                     else "Regular Chat, Multi-Modal"
                 )
                 message_embed.set_footer(
-                    text=footer_text, icon_url="https://i.imgur.com/asA13vI.png"
+                    text=footer_text, icon_url="https://i.imgur.com/wpp4TIf.png"
                 )
                 message_thread = await ctx.send(embed=message_embed)
                 thread = await message_thread.create_thread(
