@@ -11,6 +11,7 @@ import discord
 import pinecone
 from pycord.multicog import apply_multicog
 
+from cogs.code_interpreter_service_cog import CodeInterpreterService
 from cogs.search_service_cog import SearchService
 from cogs.text_service_cog import GPT3ComCon
 from cogs.image_service_cog import DrawDallEService
@@ -33,7 +34,7 @@ from services.environment_service import EnvService
 from models.openai_model import Model
 
 
-__version__ = "11.7.2"
+__version__ = "12.1.5"
 
 
 PID_FILE = Path("bot.pid")
@@ -93,7 +94,7 @@ asyncio.ensure_future(Pickler.process_pickle_queue(pickle_queue, 5, 1))
 # Settings for the bot
 #
 activity = discord.Activity(
-    type=discord.ActivityType.watching, name="for /help /gpt, and more!"
+    type=discord.ActivityType.watching, name="for /help, /gpt, and more!"
 )
 bot = discord.Bot(intents=discord.Intents.all(), command_prefix="!", activity=activity)
 usage_service = UsageService(Path(os.environ.get("DATA_DIR", os.getcwd())))
@@ -193,6 +194,14 @@ async def main():
         )
         print("The Search service is enabled.")
 
+    if EnvService.get_e2b_api_key():
+        bot.add_cog(
+            CodeInterpreterService(
+                bot, model, usage_service, deletion_queue, bot.get_cog("GPT3ComCon")
+            )
+        )
+        print("The Code Interpreter service is enabled.")
+
     bot.add_cog(
         TranscribeService(
             bot,
@@ -216,6 +225,7 @@ async def main():
             bot.get_cog("TranslationService"),
             bot.get_cog("SearchService"),
             bot.get_cog("TranscribeService"),
+            bot.get_cog("CodeInterpreterService"),
         )
     )
 
