@@ -87,6 +87,17 @@ class Settings_autocompleter:
         await ctx.interaction.response.defer()  # defer so the autocomplete in int values doesn't error but rather just says not found
         return []
 
+    async def get_function_calling_models(ctx: discord.AutocompleteContext):
+        return [
+            "gpt-4",
+            "gpt-4-32k",
+            "gpt-4-1106-preview",
+            "gpt-4-0613",
+            "gpt-3.5-turbo",
+            "gpt-3.5-turbo-1106",
+            "gpt-3.5-turbo-0613",
+        ]
+
     async def get_models(
         ctx: discord.AutocompleteContext,
     ):
@@ -111,6 +122,15 @@ class Settings_autocompleter:
         ]
         return models
 
+    async def get_dalle3_image_qualities(ctx: discord.AutocompleteContext):
+        return ["hd", "standard"]
+
+    async def get_dalle3_image_sizes(ctx: discord.AutocompleteContext):
+        return ["1024x1024", "1792x1024", "1024x1792"]
+
+    async def get_dalle3_image_styles(ctx: discord.AutocompleteContext):
+        return ["natural", "vivid"]
+
     async def get_value_moderations(
         ctx: discord.AutocompleteContext,
     ):  # Behaves a bit weird if you go back and edit the parameter without typing in a new command
