@@ -6,7 +6,7 @@ import re
 import tempfile
 import traceback
 import uuid
-from typing import Any, Tuple
+from typing import Any, Tuple, List
 
 import aiohttp
 import backoff
@@ -69,6 +69,8 @@ class Models:
     GPT4_32 = "gpt-4-32k"
     GPT4_DEV = "gpt-4-0613"
     GPT4_32_DEV = "gpt-4-32k-0613"
+    GPT_4_TURBO = "gpt-4-1106-preview"
+    GPT_4_TURBO_VISION = "gpt-4-vision-preview"
 
     # Model collections
     TEXT_MODELS = [
@@ -82,18 +84,25 @@ class Models:
         GPT4_32,
         GPT4_DEV,
         GPT4_32_DEV,
+        GPT_4_TURBO,
+        GPT_4_TURBO_VISION,
     ]
     CHATGPT_MODELS = [
         TURBO,
         TURBO_16,
         TURBO_DEV,
         TURBO_16_DEV,
+        GPT4_32,
+        GPT_4_TURBO_VISION,
+        GPT_4_TURBO,
     ]
     GPT4_MODELS = [
         GPT4,
         GPT4_32,
         GPT4_DEV,
         GPT4_32_DEV,
+        GPT_4_TURBO_VISION,
+        GPT_4_TURBO,
     ]
     EDIT_MODELS = [EDIT]
 
@@ -112,6 +121,8 @@ class Models:
         GPT4_32: 32768,
         GPT4_DEV: 8192,
         GPT4_32_DEV: 32768,
+        GPT_4_TURBO_VISION: 128000,
+        GPT_4_TURBO: 128000,
     }
 
     @staticmethod
@@ -129,13 +140,13 @@ class ImageSize:
 
 class ModelLimits:
     MIN_TOKENS = 15
-    MAX_TOKENS = 4096
+    MAX_TOKENS = 32000
 
     MIN_CONVERSATION_LENGTH = 1
     MAX_CONVERSATION_LENGTH = 100000
 
-    MIN_SUMMARIZE_THRESHOLD = 800
-    MAX_SUMMARIZE_THRESHOLD = 3500
+    MIN_SUMMARIZE_THRESHOLD = 1500
+    MAX_SUMMARIZE_THRESHOLD = 120000
 
     MIN_NUM_IMAGES = 1
     MAX_NUM_IMAGES = 4
@@ -225,7 +236,7 @@ class Model:
         self.summarize_threshold = (
             SETTINGS_DB["summarize_threshold"]
             if "summarize_threshold" in SETTINGS_DB
-            else 3000
+            else 5000
         )
         self.model_max_tokens = 4024
         self.welcome_message_enabled = (
@@ -486,6 +497,14 @@ class Model:
         self._max_tokens = Models.get_max_tokens(self._model)
         SETTINGS_DB["model"] = model
 
+        # Set the summarize threshold if the model was set to gpt-4
+        if "4-turbo" in self._model or "4-vision" in self._model:
+            self._summarize_threshold = 125000
+        elif "gpt-4" in self._model:
+            self._summarize_threshold = 28000
+        elif "gpt-3" in self._model:
+            self._summarize_threshold = 6000
+
     @property
     def max_conversation_length(self):
         return self._max_conversation_length
@@ -792,35 +811,46 @@ class Model:
 
         summary_request_text = "".join(summary_request_text)
 
-        tokens = self.usage_service.count_tokens(summary_request_text)
+        messages = []
+        messages.append(
+            {
+                "role": "system",
+                "content": summary_request_text,
+            }
+        )
 
-        async with aiohttp.ClientSession(raise_for_status=False) as session:
+        async with aiohttp.ClientSession(
+            raise_for_status=False, timeout=aiohttp.ClientTimeout(total=300)
+        ) as session:
             payload = {
-                "model": Models.DAVINCI,
-                "prompt": summary_request_text,
-                "temperature": 0.5,
-                "top_p": 1,
-                "max_tokens": self.max_tokens - tokens,
+                "model": self.model if self.model is not None else Models.GPT4_32,
+                "messages": messages,
+                "temperature": self.temp,
+                "top_p": self.top_p,
                 "presence_penalty": self.presence_penalty,
                 "frequency_penalty": self.frequency_penalty,
-                "best_of": self.best_of,
             }
             headers = {
-                "Content-Type": "application/json",
-                "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}",
+                "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}"
             }
             self.use_org = True if "true" in str(self.use_org).lower() else False
             if self.use_org:
                 if self.openai_organization:
                     headers["OpenAI-Organization"] = self.openai_organization
+
             async with session.post(
-                "https://api.openai.com/v1/completions", json=payload, headers=headers
+                "https://api.openai.com/v1/chat/completions",
+                json=payload,
+                headers=headers,
             ) as resp:
                 response = await resp.json()
-
-                await self.valid_text_request(response)
-
-                # print(response["choices"][0]["text"])
+                # print(f"Payload -> {payload}")
+                # Parse the total tokens used for this request and response pair from the response
+                await self.valid_text_request(
+                    response,
+                    model=self.model if self.model is not None else Models.GPT4_32,
+                )
+                print(f"Summary response -> {response}")
 
                 return response
 
@@ -906,6 +936,8 @@ class Model:
         Tuple[dict, bool]
     ):  # The response, and a boolean indicating whether or not the context limit was reached.
         # Validate that  all the parameters are in a good state before we send the request
+        model_selection = self.model if not model else model
+        print("The model selection is " + model_selection)
 
         # Clean up the bot name
         bot_name_clean = self.cleanse_username(bot_name)
@@ -931,7 +963,7 @@ class Model:
                     {
                         "role": "assistant",
                         "content": text,
-                    }  # TODO add back the assistant's name when the API is fixed..
+                    }
                 )
             else:
                 try:
@@ -948,9 +980,39 @@ class Model:
                     # Strip whitespace just from the right side of the string
                     text = text.rstrip()
                     text = text.replace("<|endofstatement|>", "")
-                    messages.append(
-                        {"role": "user", "name": username_clean, "content": text}
-                    )
+
+                    if "-vision" not in model_selection:
+                        messages.append(
+                            {"role": "user", "name": username_clean, "content": text}
+                        )
+
+                    else:
+                        if len(message.image_urls) > 0:
+                            messages.append(
+                                {
+                                    "role": "user",
+                                    "name": username_clean,
+                                    "content": [
+                                        {"type": "text", "text": text},
+                                    ],
+                                }
+                            )
+                            for image_url in message.image_urls:
+                                image_info = {
+                                    "type": "image_url",
+                                    "image_url": {"url": image_url, "detail": "high"},
+                                }
+                                messages[-1]["content"].append(image_info)
+                        else:
+                            messages.append(
+                                {
+                                    "role": "user",
+                                    "name": username_clean,
+                                    "content": [
+                                        {"type": "text", "text": text},
+                                    ],
+                                }
+                            )
                 except Exception:
                     text = message.text.replace("<|endofstatement|>", "")
                     messages.append({"role": "system", "content": text})
@@ -960,7 +1022,7 @@ class Model:
             raise_for_status=False, timeout=aiohttp.ClientTimeout(total=300)
         ) as session:
             payload = {
-                "model": self.model if not model else model,
+                "model": model_selection,
                 "messages": messages,
                 "stop": "" if stop is None else stop,
                 "temperature": self.temp if temp_override is None else temp_override,
@@ -972,6 +1034,11 @@ class Model:
                 if frequency_penalty_override is None
                 else frequency_penalty_override,
             }
+            if "-preview" in model_selection:
+                payload[
+                    "max_tokens"
+                ] = 4096  # TODO Not sure if this needs to be subtracted from a token count..
+
             headers = {
                 "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}"
             }
@@ -1141,8 +1208,9 @@ class Model:
             async with aiohttp.ClientSession(
                 raise_for_status=False, timeout=aiohttp.ClientTimeout(total=300)
             ) as session:
+                model_selection = self.model if not model else model
                 payload = {
-                    "model": self.model if not model else model,
+                    "model": model_selection,
                     "messages": messages,
                     "stop": "" if stop is None else stop,
                     "temperature": self.temp
@@ -1156,6 +1224,11 @@ class Model:
                     if frequency_penalty_override is None
                     else frequency_penalty_override,
                 }
+                if "preview":
+                    payload[
+                        "max_tokens"
+                    ] = 4096  # Temporary workaround while 4-turbo and vision are in preview.
+
                 headers = {
                     "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}"
                 }
@@ -1200,78 +1273,7 @@ class Model:
 
                 return response
 
-    @backoff.on_exception(
-        backoff.expo,
-        aiohttp.ClientResponseError,
-        factor=3,
-        base=5,
-        max_tries=4,
-        on_backoff=backoff_handler_http,
-    )
-    async def send_image_request(
-        self, ctx, prompt, vary=None, custom_api_key=None
-    ) -> tuple[File, list[Any]]:
-        # Validate that  all the parameters are in a good state before we send the request
-        words = len(prompt.split(" "))
-        if words < 1 or words > 75:
-            raise ValueError(
-                "Prompt must be greater than 1 word and less than 75, it is currently "
-                + str(words)
-            )
-
-        # print("The prompt about to be sent is " + prompt)
-        await self.usage_service.update_usage_image(self.image_size)
-
-        response = None
-
-        if not vary:
-            payload = {"prompt": prompt, "n": self.num_images, "size": self.image_size}
-            headers = {
-                "Content-Type": "application/json",
-                "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}",
-            }
-            self.use_org = True if "true" in str(self.use_org).lower() else False
-            if self.use_org:
-                if self.openai_organization:
-                    headers["OpenAI-Organization"] = self.openai_organization
-
-            async with aiohttp.ClientSession(
-                raise_for_status=True, timeout=aiohttp.ClientTimeout(total=300)
-            ) as session:
-                async with session.post(
-                    "https://api.openai.com/v1/images/generations",
-                    json=payload,
-                    headers=headers,
-                ) as resp:
-                    response = await resp.json()
-
-        else:
-            async with aiohttp.ClientSession(
-                raise_for_status=True, timeout=aiohttp.ClientTimeout(total=300)
-            ) as session:
-                data = aiohttp.FormData()
-                data.add_field("n", str(self.num_images))
-                data.add_field("size", self.image_size)
-                with open(vary, "rb") as f:
-                    data.add_field(
-                        "image", f, filename="file.png", content_type="image/png"
-                    )
-
-                    async with session.post(
-                        "https://api.openai.com/v1/images/variations",
-                        headers={
-                            "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}",
-                        },
-                        data=data,
-                    ) as resp:
-                        response = await resp.json()
-
-        print(response)
-
-        image_urls = []
-        for result in response["data"]:
-            image_urls.append(result["url"])
-
+    async def save_image_urls_and_return(self, image_urls, ctx):
         # For each image url, open it as an image object using PIL
         images = await asyncio.get_running_loop().run_in_executor(
             None,
@@ -1376,4 +1378,144 @@ class Model:
             image_size = os.path.getsize(temp_file.name) / 1000000
             print(f"New image size is {image_size}MB")
 
-        return (discord.File(temp_file.name), image_urls)
+        return discord.File(temp_file.name), image_urls
+
+    async def make_image_request_individual(
+        self, session, url, json_payload, headers
+    ) -> dict:
+        async with session.post(url, json=json_payload, headers=headers) as resp:
+            return await resp.json()
+
+    @backoff.on_exception(
+        backoff.expo,
+        aiohttp.ClientResponseError,
+        factor=3,
+        base=5,
+        max_tries=4,
+        on_backoff=backoff_handler_http,
+    )
+    async def send_image_request(
+        self, ctx, prompt, quality, image_size, style, custom_api_key=None
+    ) -> tuple[File, List[Any]]:
+        words = len(prompt.split(" "))
+        if words < 1 or words > 75:
+            raise ValueError(
+                f"Prompt must be greater than 1 word and less than 75, it is currently {words}"
+            )
+
+        await self.usage_service.update_usage_image(image_size)
+
+        image_urls = []
+        tasks = []
+        payload = {
+            "prompt": prompt,
+            "quality": quality,
+            "style": style,
+            "model": "dall-e-3",
+            "size": image_size,
+        }
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}",
+        }
+        self.use_org = "true" in str(self.use_org).lower()
+        if self.use_org and self.openai_organization:
+            headers["OpenAI-Organization"] = self.openai_organization
+
+        # Setup the client session outside of the loop
+        async with aiohttp.ClientSession(
+            raise_for_status=True, timeout=aiohttp.ClientTimeout(total=300)
+        ) as session:
+            # Create a coroutine for each image request and store it in the tasks list
+            for _ in range(self.num_images):
+                task = self.make_image_request_individual(
+                    session,
+                    "https://api.openai.com/v1/images/generations",
+                    payload,
+                    headers,
+                )
+                tasks.append(task)
+
+            # Run all tasks in parallel and wait for them to complete
+            responses = await asyncio.gather(*tasks)
+
+            # Process the results
+            for response in responses:
+                print(response)
+                for result in response["data"]:
+                    image_urls.append(result["url"])
+
+        # Now all the requests are done, we can save the URLs
+        return await self.save_image_urls_and_return(image_urls, ctx)
+
+    @backoff.on_exception(
+        backoff.expo,
+        aiohttp.ClientResponseError,
+        factor=3,
+        base=5,
+        max_tries=4,
+        on_backoff=backoff_handler_http,
+    )
+    async def send_image_request_old(
+        self, ctx, prompt, vary=None, custom_api_key=None
+    ) -> tuple[File, list[Any]]:
+        # Validate that  all the parameters are in a good state before we send the request
+        words = len(prompt.split(" "))
+        if words < 1 or words > 75:
+            raise ValueError(
+                "Prompt must be greater than 1 word and less than 75, it is currently "
+                + str(words)
+            )
+
+        # print("The prompt about to be sent is " + prompt)
+        await self.usage_service.update_usage_image(self.image_size)
+
+        response = None
+
+        if not vary:
+            payload = {"prompt": prompt, "n": self.num_images, "size": self.image_size}
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}",
+            }
+            self.use_org = True if "true" in str(self.use_org).lower() else False
+            if self.use_org:
+                if self.openai_organization:
+                    headers["OpenAI-Organization"] = self.openai_organization
+
+            async with aiohttp.ClientSession(
+                raise_for_status=True, timeout=aiohttp.ClientTimeout(total=300)
+            ) as session:
+                async with session.post(
+                    "https://api.openai.com/v1/images/generations",
+                    json=payload,
+                    headers=headers,
+                ) as resp:
+                    response = await resp.json()
+
+        else:
+            async with aiohttp.ClientSession(
+                raise_for_status=True, timeout=aiohttp.ClientTimeout(total=300)
+            ) as session:
+                data = aiohttp.FormData()
+                data.add_field("n", str(self.num_images))
+                data.add_field("size", self.image_size)
+                with open(vary, "rb") as f:
+                    data.add_field(
+                        "image", f, filename="file.png", content_type="image/png"
+                    )
+
+                    async with session.post(
+                        "https://api.openai.com/v1/images/variations",
+                        headers={
+                            "Authorization": f"Bearer {self.openai_key if not custom_api_key else custom_api_key}",
+                        },
+                        data=data,
+                    ) as resp:
+                        response = await resp.json()
+
+        image_urls = []
+        for result in response["data"]:
+            image_urls.append(result["url"])
+
+        return await self.save_image_urls_and_return(image_urls, ctx)
