@@ -49,7 +49,10 @@ class UsageService:
         return UsageService.MODEL_COST_MAP.get(model, "davinci")
 
     async def get_price(self, tokens_used, mode: ModeType = None):
-        tokens_used = int(tokens_used)
+        if isinstance(tokens_used, str) or isinstance(tokens_used, int):
+            tokens_used = int(tokens_used)
+        else:
+            tokens_used = int(len(tokens_used))
         price = (tokens_used / 1000) * await self.get_model_cost(
             mode
         )  # This is a very rough estimate
@@ -89,16 +92,17 @@ class UsageService:
         return len(res)
 
     async def update_usage_image(self, image_size):
-        # 1024×1024    $0.020 / image
-        # 512×512    $0.018 / image
-        # 256×256    $0.016 / image
-
+        image_size = image_size.split(" ")[0]
         if image_size == "1024x1024":
-            price = 0.02
+            price = 0.04
         elif image_size == "512x512":
             price = 0.018
         elif image_size == "256x256":
             price = 0.016
+        elif image_size == "1792x1024":
+            price = 0.08
+        elif image_size == "1024x1792":
+            price = 0.08
         else:
             raise ValueError("Invalid image size")
 
